@@ -10,14 +10,19 @@
 """
 
 from .pluginkit import PluginManager, push_dcp
-from .utils import Flask, LocalStorage, RedisStorage, MongoStorage, \
-    JsonResponse
+from .utils import (
+    Flask,
+    LocalStorage,
+    RedisStorage,
+    MongoStorage,
+    JsonResponse,
+)
 from ._installer import PluginInstaller
 from ._web import blueprint
 
 __author__ = "staugur <me@tcw.im>"
 
-__version__ = "3.6.0"
+__version__ = "3.6.1"
 
 __all__ = [
     "Flask",
